from enum import Enum


class MMRVersions(str, Enum):
    v1 = "v1"
    v2 = "v2"


class FeaturedItemsVersion(str, Enum):
    v1 = "v1"
    v2 = "v2"


class LeaderboardVersions(str, Enum):
    v1 = "v1"
    v2 = "v2"
    v3 = "v3"


class AccountVersion(str, Enum):
    v1 = "v1"
    v2 = "v2"
