from enum import Enum


class CCRegions(str, Enum):
    en_gb = "en-gb"
    en_us = "en-us"
    es_es = "es-es"
    es_mx = "es-mx"
    fr_fr = "fr-fr"
    it_it = "it-it"
    ja_jp = "ja-jp"
    ko_kr = "ko-kr"
    pt_br = "pt-br"
    ru_ru = "ru-ru"
    tr_tr = "tr-tr"
    vi_vn = "vi-vn"


class Locales(str, Enum):
    ar_AE = "ar-AE"
    de_DE = "de-DE"
    en_GB = "en-GB"
    en_US = "en-US"
    es_ES = "es-ES"
    es_MX = "es-MX"
    fr_FR = "fr-FR"
    id_ID = "id-ID"
    it_IT = "it-IT"
    ja_JP = "ja-JP"
    ko_KR = "ko-KR"
    pl_PL = "pl-PL"
    pt_BR = "pt-BR"
    ru_RU = "ru-RU"
    th_TH = "th-TH"
    tr_TR = "tr-TR"
    vi_VN = "vi-VN"
    zn_CN = "zn-CN"
    zn_TW = "zn-TW"
